from test.test_builder import *
from test.test_encoding import *
from test.test_loader import *
from test.test_plugins_helloworld import *
from test.test_plugins_kanjidic2 import *
from test.test_plugins_kanjistream import *
from test.test_plugins_jsonstream import *
from test.test_plugins_mergedict import *
from test.test_sample_generatedb import *
from test.test_service import *
