# -*- coding: utf-8 -*-

# kanjidb's package version information
__version_major__ = "0.1"
__version__ = "{}a2".format(__version_major__)
__version_long__ = "{}a2".format(__version_major__)
__status__ = "Alpha"
__author__ = "Jeremy Morosi"
__author_email__ = "jeremymorosi@hotmail.com"
__url__ = "https://github.com/Nauja/kanjidb"
