from . import __version__ as version_info
from .__version__ import __version_major__, __version_long__, __version__, __status__


from kanjidb.main import *

__all__ = ["help", "main", "__version__", "version_info"]
